<?php

require_once(dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php");
require_once("lib-action/action.php");

class FirstTimeDisplayStep extends MB_Action
{
	public function execute()
	{
		parent::execute();
		$action = bgpc("a");
		switch ($action) {
			case "company":
				$this->showCompany();
				break;
			case "registrar":
				$this->showRegistrar();
				break;
			case "gateway":
				$this->showGateway();
				break;
			case "plan":
				$this->showPlan();
				break;
			case "ip":
				$this->showIP();
				break;
			case "taxes":
				$this->showTaxes();
				break;
			case "fraudguardian":
				$this->showFraudguardian();
				break;
			case "encryption":
				$this->showEncryption();
				break;

			case "finished":
				$this->smartyVars["step"] = "finished";
				break;
			default:
				redirect("configs.php", "FirstTimeIntroduction");
				break;
		}

		$this->addTemplate("configs/FirstTimeDisplayStep");
		return $this->hasExecutedSuccessfully();
	}


	private function showCompany()
	{
		$this->mergeCountryData();
		require_once("object_managers/CompanyManager.php");
		$this->smartyVars["company"] = CompanyManager::getProviderCompany();
		$this->smartyVars["step"] = "company";
	}

	private function showTaxes()
	{
		require_once("object_managers/CompanyManager.php");
		$company = CompanyManager::getProviderCompany();
		$country = $company->Country;
		$this->smartyVars["providerCountry"] = blmsg("TRANS_ISO_3166_{$country->countries_iso_2}");

		$this->smartyVars["defaultGrossPriceDisplay"] = $country->default_gross_price_display;

		require_once("object_managers/OrderFormManager.php");
		$order_form = OrderFormManager::getDefaultForm();
		$this->smartyVars["requestTaxVatID"] = $order_form->request_tax_vat_id;
		$this->smartyVars["orderFormVerifyVat"] =  $order_form->order_form_verify_vat;
		if ( OrderFormManager::isVatCountry($country->countries_iso_2) ) {
			$this->smartyVars["requestTaxVatID"] = $this->smartyVars["orderFormVerifyVat"] =  1;
		}
		$taxZoneGroups = arrayFor(":taxZoneGroups");

		if ($taxZoneGroups[0]['taxZoneGroupID']){
			$this->smartyVars["taxZoneGroupID"] = $taxZoneGroups[0]['taxZoneGroupID'];
			$this->smartyVars["taxZone"] = oneOf(":taxZones", array(
				"taxZoneGroupID" => $taxZoneGroups[0]['taxZoneGroupID'],
			));
		}


		$this->smartyVars["products"] = optionsFor(":products");
		$productVariants = arrayFor(":productVariants", array(
			"productVariantActive" => 1,
		));

		$this->smartyVars["taxesEnabled"] = 0;
		$this->smartyVars["productVariants"] = array();
		foreach ($productVariants as $v){
			if ($v["taxZoneGroupID"]==$this->smartyVars["taxZoneGroupID"]){
				$this->smartyVars["taxesEnabled"] = 1;
			}
			$this->smartyVars["productVariants"][$v["productID"]][] = $v;
		}

		$this->smartyVars["step"] = "taxes";
	}

	private function showRegistrar()
	{
		global $CONF;
		if (licRawData!="FREE"){
			$ama = $_MB_ALIASES["admin"] ? $_MB_ALIASES["admin"] : "admin";
			$returnURL = (($CONF["secureURLProtocol"]=="http")?"http://".$CONF["standardURL"]:"https://".$CONF["secureURL"])."/".$ama."/configs.php?action=".getActionID("FirstTimeDisplayStep")."&sid=".$_REQUEST["sid"]."&a=registrar&enomUsername=USERNAME&enomPassword=PASSWORD";
			$securityCode = getPasswordHash(Crypt::ENCRYPTION_KEY.$returnURL);
			$this->smartyVars["returnURL"] = base64_encode($returnURL);
			$this->smartyVars["securityCode"] = $securityCode;
			$this->smartyVars["adminInfo"] = oneByID(":admins", $CONF["user"]["id"]);
			$this->smartyVars["companyInfo"] = oneByID(":companies", 1);
			$country = oneByID(":countries", $this->smartyVars["companyInfo"]["countriesID"]);
			$this->smartyVars["companyInfo"]["countriesISO2"] = $country["countriesISO2"];
		}

		$availableRegistrars = getAvailableModules("registrar");
		$registrarConfigGroup = Factory()->ConfigGroup(0)->getChild("domain")->getChild("registrar");
		$existingRegistars = array();
		foreach(Factory()->ConfigGroupList(array("parent_config_group_id" => $registrarConfigGroup->config_group_id)) as $registrar) {
			if (0 == preg_match("/pbaregistrar/i", $registrar->config_group_name)) {
				$existingRegistars[] = $registrar->config_group_name;
			}
		}

		foreach ($availableRegistrars as $regName => $regFile) {
			if ($regName=="generic") continue;
			if(!in_array($regName, $existingRegistars)) {
				$addNew = new MBQuery("SetConfigGroups", "insert", array(
					"parentConfigGroupID" => $registrarConfigGroup->config_group_id,
					"addRegistrar" => $regName,
				));
				$addNew->getResult();
			}
		}

		// if user tryed to get eNom account, save his username and password
		if (isset($_REQUEST["enomUsername"])) {
			$enom = Factory()->ConfigGroup(0)->getChild("domain")->getChild("registrar")->getChild("enom");
			$cpu = oneOf(":configParams", array(
				"configParamName" => "moduleUsername",
				"configGroupID" => $enom->config_group_id,
			));
			updateForKey(":configParams", $cpu["configParamID"], array(
				"configParamValue" => bgpc("enomUsername"),
			));
			$cpp = oneOf(":configParams", array(
				"configParamName" => "modulePassword",
				"configGroupID" => $enom->config_group_id,
			));
			updateForKey(":configParams", $cpp["configParamID"], array(
				"configParamValue" => getEncryptedText(bgpc("enomPassword")),
			));
		}

		unset($CONF["domain"]["registrar"]["generic"]);

		require_once("ModuleTool.php");
		$registrarControls = array();
		foreach($registrarConfigGroup->ConfigGroupList() as $config_group) {
			$registrar = Factory()->BillingRegistrar($config_group->id);
			if (0 < preg_match("/pbaregistrar/i", $registrar->module_name)) {
				continue;
			}
			$controls = array();
			foreach ( ModuleTool::getModuleControls("registrar", $registrar->module_name) as $inputPanel ) {
				$inputPanel->initFromArray(DataObject::getMapOf($registrar->ConfigParams, "name", "value"));
				foreach ( $inputPanel->getInputControls() as $input_control ) {
					if("moduleEnabled" == $input_control->name) {
						continue;
					}

					$input_control->name = "registrars_{$registrar->id}_{$input_control->name}";
					$controls[] = array("control" => $input_control, "group" => $inputPanel->title);
				}
			}

			$registrarControls[] = array(
			    "configGroupID" => $registrar->id,
			    "configGroupName" => $registrar->module_name,
			    "moduleEnabled" => $registrar->ConfigParams["moduleEnabled"]->value,
			    "configGroupTransKey" => $registrar->name,
			    "inputControls" => $controls
			);
		}

		$this->smartyVars["registrars"] = $registrarControls;
		$this->smartyVars["step"] = "registrar";
	}

	private function showGateway()
	{
		require_once("object_managers/OrderFormManager.php");
		$default_form = OrderFormManager::getDefaultForm();
		require_once("object_managers/CompanyManager.php");
		$company = CompanyManager::getProviderCompany();
		$this->smartyVars["providerCountry"] = blmsg("TRANS_ISO_3166_{$company->Country->countries_iso_2}");
		$enabled_gateways = array();
		if ( !is_null($default_form->PrimaryGateway) ) {
			$enabled_gateways["{$default_form->PrimaryGateway->payment_gateway_module}{$default_form->PrimaryGateway->id}"] = 1;
		}
		foreach ( $default_form->ThirdPartyGatewaysList() as $gateway ) {
			$enabled_gateways["{$gateway->payment_gateway_module}{$gateway->id}"] = 1;
		}

		require_once("object_managers/PaymentGatewayManager.php");
		$this->smartyVars["primaryGateways"] = $this->getPaymentGatewaysVars(
			PaymentGatewayManager::getModules(PaymentGateway::TYPE_PRIMARY)
		);
		$this->smartyVars["thirdPartyGateways"] = $this->getPaymentGatewaysVars(
			PaymentGatewayManager::getModules(PaymentGateway::TYPE_THIRD_PARTY)
		);
		$this->smartyVars["enabledGateways"] = $enabled_gateways;
		$this->smartyVars["step"] = "gateway";

		require_once("HSPC/Tools.php");
		$status = HSPCTools::getConnectionStatus();
		if (!$status["available"]) {
			return;
		}

		$hspcCreditCardGateways = PaymentGatewayManager::getHSPCModules(PaymentGateway::TYPE_HSPC_CREDIT_CARD);
		foreach ($hspcCreditCardGateways as $gateway) {
			if ("Dummy" == $gateway["name"]) {
				continue;
			}
			$this->smartyVars["hspcCreditCardGateways"][$gateway["name"]] = array("name" => $gateway["name"], "description" => "TRANS_WIZARD_HSPC_CREDIT_CARD_DESCR");
		}
		$hspcThirdPartyGateways = PaymentGatewayManager::getHSPCModules(PaymentGateway::TYPE_HSPC_THIRD_PARTY);
		foreach ($hspcThirdPartyGateways as $thirdHSCPgateway) {
			$this->smartyVars["hspcThirdPartyGateways"][$thirdHSCPgateway["name"]] = array("name" => $thirdHSCPgateway["name"], "description" =>"TRANS_WIZARD_HSPC_THIRD_PARTY_DESCR");
		}
	}

	private function showEncryption()
	{
		require_once("object_managers/CryptManager.php");
		if ( CryptManager::isEncryptionEnabled() ) {
			// we don't want to mess with encryption if they have already done it
			$this->showFraudguardian();
			return;
		}
		$this->smartyVars["encryption"]["encryption_agree"] = "agree";
		$this->smartyVars["step"] = "encryption";
	}

	private function showFraudguardian()
	{
		$fraudguardian = Factory()->ConfigGroup(0)->getChild("securitySettings")->getChild("fraudguardian")->getChild("0");
		$this->smartyVars["usernameFraudGuardian"] = $fraudguardian->ConfigParams["usernameFraudGuardian"]->value;
		$this->smartyVars["passwordFraudGuardian"] = $fraudguardian->ConfigParams["passwordFraudGuardian"]->value;
		$this->smartyVars["enableFraudGuardian"] = $fraudguardian->ConfigParams["enableFraudGuardian"]->value;
		$this->smartyVars["step"] = "fraudguardian";
	}

	private function showPlan()
	{
		require_once("object_managers/ControlPanelManager.php");
		if (is_null($masterPanel = ControlPanelManager::getMasterPanel())) {
			$this->showTaxes();
			return;
		}
		require_once("object_managers/ProductManager.php");
		$notConfiguedPlans = ProductManager::getNotConfiguedPlans($masterPanel);
		if (empty($notConfiguedPlans)) {
			$this->showIP();
			return;
		}
		$this->smartyVars["notConfiguedPlans"] = $notConfiguedPlans;
		$this->smartyVars["step"] = "plan";
	}

	private function showIP()
	{
		require_once("object_managers/ControlPanelManager.php");
		if (is_null($masterPanel = ControlPanelManager::getMasterPanel())) {
			$this->showTaxes();
			return;
		}
		$requiredIPs = array("ipv4Address" => array("shared" => 0, "dedicated" => 0), "ipv6Address" => array("shared" => 0, "dedicated" => 0));
		$requiredIPs["ipv4Address"][$this->smartyVars["ipv4Address"] = bgpc("ipv4Address")] = 1;
		$requiredIPs["ipv6Address"][$this->smartyVars["ipv6Address"] = bgpc("ipv6Address")] = 1;
		foreach ($masterPanel->ServerGroup->ProductList() as $plan) {
			if ($plan->isAddon()) {
				continue;
			}
			$planAttributes = reset($plan->ProductAttributesList(array("server_group_id" => $masterPanel->server_group_id)));
			$requiredIPs["ipv4Address"][$planAttributes["ipv4Address"]->value] = 1;
			$requiredIPs["ipv6Address"][$planAttributes["ipv6Address"]->value] = 1;
		}
		if (1 == $requiredIPs["ipv4Address"]["shared"] && "" != $masterPanel->ConfigParams["ip"]->value) {
			$requiredIPs["ipv4Address"]["shared"] = 0;
		}
		require_once("object_managers/IPAddressManager.php");
		if (1 == $requiredIPs["ipv4Address"]["dedicated"] && 0 < count($masterPanel->IPLogList(array("ip_version" => IPAddressManager::IPv4)))) {
			$requiredIPs["ipv4Address"]["dedicated"] = 0;
		}
		if (1 == $requiredIPs["ipv6Address"]["shared"] && "" != $masterPanel->ConfigParams["ipv6"]->value) {
			$requiredIPs["ipv6Address"]["shared"] = 0;
		}
		if (1 == $requiredIPs["ipv6Address"]["dedicated"] && 0 < count($masterPanel->IPLogList(array("ip_version" => IPAddressManager::IPv6)))) {
			$requiredIPs["ipv6Address"]["dedicated"] = 0;
		}
		if (0 == $requiredIPs["ipv4Address"]["shared"] && 0 == $requiredIPs["ipv4Address"]["dedicated"] && 0 == $requiredIPs["ipv6Address"]["shared"]  && 0 == $requiredIPs["ipv6Address"]["dedicated"]) {
			redirect("configs.php", "FirstTimePerformAction", array("a" => "ip"), array("ipv4Address" => $this->smartyVars["ipv4Address"], "ipv6Address" => $this->smartyVars["ipv6Address"]));
		}
		$this->smartyVars["requiredIPs"] = $requiredIPs;
		$this->smartyVars["step"] = "ip";
		$this->smartyVars["invalid"] = bgpc("invalid", array());
		foreach (array("ip", "ipMax", "ipv6", "ipv6Max") as $name) {
			$this->smartyVars["dedicatedIPs"][$name] = bgpc($name);
		}
		if (0 == $requiredIPs["ipv4Address"]["shared"] && 0 == $requiredIPs["ipv6Address"]["shared"]) {
			return;
		}

		try {
			require_once("PleskAPIFactory.php");
			$api = PleskAPIFactory::getInstance(array("1.6.3.2", "1.6.3.0"), $masterPanel);
			require_once("Plesk/IPAddressAPI.php");
			$result = $api->ip()->get();
			if (!is_null($result->error)) {
				throw new $result->error;
			}
			require_once("object_managers/IPAddressManager.php");
			$this->smartyVars["ipv4Addresses"] = $this->smartyVars["ipv6Addresses"] = array();
			foreach ($result->data as $panelIPAddress) {
				if (PleskIPAddress::TYPE_SHARED != $panelIPAddress->type) {
					continue;
				}
				switch (IPAddressManager::getVersion($panelIPAddress->ip_address)) {
				case IPAddressManager::IPv4:
					$this->smartyVars["ipv4Addresses"][$panelIPAddress->ip_address] = $panelIPAddress->ip_address;
					break;
				case IPAddressManager::IPv6:
					$this->smartyVars["ipv6Addresses"][$panelIPAddress->ip_address] = $panelIPAddress->ip_address;
					break;
				}
			}
			if (empty($this->smartyVars["ipv4Addresses"]) && 1 == $requiredIPs["ipv4Address"]["shared"]) {
				$this->smartyVars["warningMessages"][] = blmsg("TRANS_FIRST_TIME_PANEL_SHARED_IPV4_NOT_FOUND_MSG", array("PANEL_NAME" => $masterPanel->name));
				$this->smartyVars["ipv4Addresses"] = array("" => blmsg("TRANS_NONE"));
			}
			if (empty($this->smartyVars["ipv6Addresses"]) && 1 == $requiredIPs["ipv6Address"]["shared"]) {
				$this->smartyVars["warningMessages"][] = blmsg("TRANS_FIRST_TIME_PANEL_SHARED_IPV6_NOT_FOUND_MSG", array("PANEL_NAME" => $masterPanel->name));
				$this->smartyVars["ipv6Addresses"] = array("" => blmsg("TRANS_NONE"));
			}
			$this->smartyVars["sharedip"] = bgpc("sharedip", $masterPanel->ConfigParams["ip"]->value);
			$this->smartyVars["sharedipv6"] = bgpc("sharedipv6", $masterPanel->ConfigParams["ipv6"]->value);
		} catch (PleskAPIException $e) {
			$this->smartyVars["errorMessages"][] = blmsg("TRANS_FIRST_TIME_PANEL_IP_GET_ERROR_MSG", array("PANEL_NAME" => $masterPanel->name, "MESSAGE" => $e->getMessage()));
		}
	}


	private function getPaymentGatewaysVars($modules)
	{
		$gateways_vars = array();
		$gatewayStatuses = PaymentGateway::statuses();
		$this->smartyVars["gatewayStatusTesting"] = PaymentGateway::STATUS_TESTING;
		foreach ( $modules as $module_name ) {
			// add item for new payment gateway
			$gateways_vars[$module_name] = $this->getPaymentGatewayVars($module_name, $gatewayStatuses);

			// add items for exisiting payment gateways
			foreach ( Factory()->PaymentGatewayList(array("payment_gateway_module" => $module_name)) as $gateway ) {
				$gateways_vars["{$module_name}{$gateway->id}"] = $this->getPaymentGatewayVars($module_name, $gatewayStatuses, $gateway);
			}
		}
		if ( false === uasort($gateways_vars, array($this, "comparePaymentGatewayDisplayName"))) {
			global $php_errormsg;
			throw new Exception("usort() failed: {$php_errormsg}");
		}
		return $gateways_vars;
	}

	private function getPaymentGatewayVars($module_name, $gatewayStatuses, $gateway = null)
	{
		$module_info = PaymentGatewayManager::getModuleInfo($module_name);
		$gateway_vars["name"] = (PaymentGateway::STATUS_STABLE < $module_info["status"]) ? "{$module_info['displayName']} (".blmsg($gatewayStatuses[$module_info["status"]]).")" : $module_info['displayName'] ;
		$gateway_vars["status"] = $module_info["status"];
		$gateway_vars["countries"] = isset($module_info["countryCodes"]) ? $this->getModuleCountryNames($module_info["countryCodes"]) : "" ;
		if ( !is_null($gateway) ) {
			$gateway_vars["name"] .= " - {$gateway->payment_gateway_name}";
		}
		$config_name = (is_null($gateway) ? "{$module_name}" : "{$module_name}{$gateway->id}");
		$input_gateway_name = UIFactory()->InputString("{$config_name}_payment_gateway_name", blmsg("TRANS_GATEWAY_NAME"), true);
		$input_gateway_name->value = (is_null($gateway) ? $module_info["displayName"] : $gateway->payment_gateway_name);
		$gateway_vars["inputControls"][] = array("control" => $input_gateway_name, "group" => "");
		require_once("ModuleTool.php");
		foreach ( ModuleTool::getModuleControls("gateway", $module_name) as $control ) {
			if ( !is_null($gateway) ) {
				require_once("DataObject.php");
				$control->initFromArray(DataObject::getMapOf($gateway->PaymentGatewayParams, "name", "value"));
			}
			foreach ( $control->getInputControls() as $input_control ) {
				$input_control->name = "{$config_name}_{$input_control->name}";
				$gateway_vars["inputControls"][] = array("control" => $input_control, "group" => "");
			}
		}
		return $gateway_vars;
	}

	private function getModuleCountryNames($countryCodes)
	{
		$countryNames = array();
		foreach ($countryCodes as $countryCode) {
			$countryNames[] = blmsg("TRANS_ISO_3166_{$countryCode}");
		}
		return implode(", ", $countryNames);
	}

    private function comparePaymentGatewayDisplayName($leftGatewayVars, $rightGatewayVars)
    {
		if ($leftGatewayVars['status'] == $rightGatewayVars['status'])
			return strcasecmp($leftGatewayVars["name"], $rightGatewayVars["name"]);
   		return ($leftGatewayVars['status'] < $rightGatewayVars['status']) ? -1 : 1;

    }
}
