<?php
require_once(dirname(__FILE__)."/../../../../lib-action/action.php");
require_once("UI/Page.php");
require_once("UI/PageController.php");

class AddDomainRegistrarSelectModulePage extends UIPage
{
	public function createControls()
	{
		$this->title = blmsg("TRANS_ADD_DOMAIN_REGISTRAR");
		$this->subtitle = blmsg("TRANS_ADD_DOMAIN_REGISTRAR_TITLE");
		$this->subtitle_description = blmsg("TRANS_ADD_DOMAIN_REGISTRAR__CHOOSE_MODULE_DESC");
		$this->subtitle_icon_id = "bid-domain";

		$this->input_module = UIFactory()->InputDropDown("module", blmsg("TRANS_DOMAIN_REGISTRAR_PROVIDER_TITLE"));

		$input_panel = UIFactory()->InputPanel();
		$input_panel->addInputControl($this->input_module);

		$this->addInputPanel($input_panel);
	}

	public function initControl()
	{
		require_once("object_managers/BillingRegistrarManager.php");

		foreach (BillingRegistrarManager::getNotInstalledModules() as $module) {
			$module_info = BillingRegistrarManager::getModuleInfo($module);
			$this->input_module->addOption($module, $module_info["displayName"]);
		}
		parent::initControl();
	}

	private $input_module;
}

class AddDomainRegistrarParamsPage extends UIPage
{
	public function createControls()
	{
		$module = $this->getValue("module");

		$this->title = blmsg("TRANS_ADD_DOMAIN_REGISTRAR_MODULE", array("MODULE" => $module));
		$this->subtitle = blmsg("TRANS_ADD_DOMAIN_REGISTRAR_TITLE");
		$this->subtitle_description = blmsg("TRANS_ADD_DOMAIN_REGISTRAR__CONFIGURE_SETTINGS_DESC");
		$this->subtitle_icon_id = "bid-domain";

		// Panel for the general settings
		$this->addInputPanel($generalInputPanel = UIFactory()->InputPanel());

		// Registrar name
		require_once("object_managers/BillingRegistrarManager.php");
		$this->input_registrar_name = UIFactory()->InputString("registrar_name", blmsg("TRANS_NAME"), true,
			array("BillingRegistrarManager", "checkName", $registrar->id)
		);
		$module_info = BillingRegistrarManager::getModuleInfo($module);
		$this->input_registrar_name->value = $module_info["displayName"];
		$generalInputPanel->addInputControl($this->input_registrar_name);

		// Registrar activity status
		$this->input_registrar_enabled = UIFactory()->InputCheckBox("registrar_enabled", blmsg("TRANS_ENABLED"));
		$this->input_registrar_enabled->value = false;
		$generalInputPanel->addInputControl($this->input_registrar_enabled);

		$this->module_input_controls = array();
		foreach (ModuleTool::getModuleControls("registrar", $module) as $moduleInputPanel) {
			$this->module_input_controls = array_merge($this->module_input_controls, $moduleInputPanel->getInputControls());
			if ( "" == $moduleInputPanel->title ) {
				$moduleInputPanel->title = blmsg("TRANS_CONFIGURATION_PARAMETERS");
			}
			$this->addInputPanel($moduleInputPanel);
		}
	}

	public function processInput()
	{
		$registrar = Factory()->BillingRegistrar();
		$registrar->name = $this->input_registrar_name->value;
		$registrar->module_name = $this->getValue("module");
		$registrar->ConfigParams["moduleEnabled"]->value = $this->input_registrar_enabled->value;

		$collect_input = array();
		foreach ($this->module_input_controls as $control) {
			$collect_input[$control->name] = $control->value;
		}
		$module = BillingRegistrarManager::getModuleInstance($registrar->module_name);
		$collect_output = $module->collectConfigData($collect_input);

		foreach ($this->module_input_controls as $control) {
			$value = $collect_output["data"][$control->name];
			switch ($control->name) {
			case "modulePassword":
				require_once("lib-tk/include/crypt/crypt.php");
				if ("" != $value)
				{
					require_once("lib-tk/include/crypt/crypt.php");
					$registrar->ConfigParams[$control->name]->value = getEncryptedText(getStuff($value));
				}
				break;
			default:
				$registrar->ConfigParams[$control->name]->value = $value;
			}
		}
		try {
			$registrar->update();
			$registrar->ConfigParams->update();

			foreach($registrar->getModule()->capabilities[REGISTRAR_TLDS] as $tld) {
				try {
					$system_tld = Factory()->TLD($tld, "tld_name");
				} catch (ProductException $e) {
					if (ProductException::E_OBJECT_NOT_FOUND == $e->getCode()) {
						continue;
					}
					throw $e;
				}
				$new_system_tld = Factory()->TLD();
				$new_system_tld->tld_name = $tld;
				$new_system_tld->tld_extension = $tld;
				$new_system_tld->tld_active = 0;
				$new_system_tld->registrar_id = $registrar->id;
				$new_system_tld->update();
			}
			getCurrentSession()->Storage->infoMessages[] = blmsg("TRANS_SETTINGS_SAVED_SUCCESSFULLY");
		} catch (ProductException $e) {
			$this->addErrorMessage(UIFactory()->ErrorMessage($e->getMessage()));
			return false;
		}
		return true;
	}

	private function addModuleControls($module)
	{
		$this->module_input_controls = array();
		require_once("ModuleTool.php");
		foreach ( ModuleTool::getModuleControls("registrar", $module) as $control ) {
			$this->module_input_controls = array_merge($this->module_input_controls, $control->getInputControls());
			if ( "" == $control->title ) {
				$control->title = blmsg("TRANS_DOMAIN_REGISTRAR_SETTINGS_TITLE");
			}
			$this->addInputPanel($control);
		}
	}

	private $module_input_controls;
	private $input_registrar_name;
	private $input_registrar_enabled;
}


class AddDomainRegistrar extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$back_url = action_url("registrars", "ShowRegistrars");
		$controller  = UIFactory()->WizardController($back_url);
		$controller->addPage($select_module_page = new AddDomainRegistrarSelectModulePage($controller));
		$controller->addPage($module_params_page = new AddDomainRegistrarParamsPage($controller));
		$select_module_page->parent_title = $module_params_page->parent_title = blmsg("TRANS_DOMAIN_REGISTRARS_TITLE");
		$select_module_page->parent_url = $module_params_page->parent_url = $back_url;
		$controller->processRequest();

		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}
