<?php
require_once("lib-action/action.php");
require_once("UI/Page.php");

class EditDomainRegistrarPage extends UIPage
{
	public function createControls()
	{
		require_once("ModuleTool.php");
		require_once("object_managers/BillingRegistrarManager.php");

		$registrar = Factory()->BillingRegistrar($this->getValue("configGroupID"));

		// Panel for the general settings
		$this->addInputPanel($generalInputPanel = UIFactory()->InputPanel());

		// Registrar name
		$this->input_registrar_name = UIFactory()->InputString("registrar_name", blmsg("TRANS_NAME"), true,
			array("BillingRegistrarManager", "checkName", $registrar->id)
		);
		$this->input_registrar_name->value = $registrar->name;
		$generalInputPanel->addInputControl($this->input_registrar_name);

		// Registrar activity status
		$this->input_registrar_enabled = UIFactory()->InputCheckBox("registrar_enabled", blmsg("TRANS_ENABLED"));
		$this->input_registrar_enabled->value = $registrar->ConfigParams["moduleEnabled"]->value;
		$generalInputPanel->addInputControl($this->input_registrar_enabled);

		$this->module_input_controls = array();
		foreach (ModuleTool::getModuleControls("registrar", $registrar->module_name, $registrar->id) as $moduleInputPanel) {
			$moduleInputPanel->initFromArray(DataObject::getMapOf($registrar->ConfigParams, "name", "value"));
			$this->module_input_controls = array_merge($this->module_input_controls, $moduleInputPanel->getInputControls());
			if ( "" == $moduleInputPanel->title ) {
				$moduleInputPanel->title = blmsg("TRANS_CONFIGURATION_PARAMETERS");
			}
			$this->addInputPanel($moduleInputPanel);
		}
	}

	public function initControl()
	{
		$registrar = Factory()->BillingRegistrar($this->getValue("configGroupID"));
		if ("wildwestdomains" == $registrar->module_name &&
			1 == $registrar->ConfigParams["moduleEnabled"]->value &&
			0 == $registrar->ConfigParams["moduleCertified"]->value
		) {
			$this->addWarningMessage(UIFactory()->WarningMessage(blmsg("TRANS_REGISTRAR_WILDWEST_CERTIFICATION_REQUIRED_MSG", array(
				"URL" => action_url("modules", "CertifyGoDaddyAccount", array("configGroupID" => $registrar->id))
			))));
		}

		parent::initControl();
	}

	public function processInput()
	{
		$registrar = Factory()->BillingRegistrar($this->getValue("configGroupID"));
		$registrar->name = $this->input_registrar_name->value;
		$registrar->ConfigParams->clear();
		$registrar->ConfigParams["moduleEnabled"]->value = $this->input_registrar_enabled->value;

		$collect_input = array();
		foreach ($this->module_input_controls as $control) {
			$collect_input[$control->name] = $control->value;
		}
		$module = $registrar->getModule();
		$collect_output = $module->collectConfigData($collect_input);

		foreach ($this->module_input_controls as $control) {
			$value = $collect_output["data"][$control->name];
			switch ($control->name)
			{
				case "modulePassword":
					require_once("lib-tk/include/crypt/crypt.php");
					if ("" != $value)
					{
						require_once("lib-tk/include/crypt/crypt.php");
						$registrar->ConfigParams[$control->name]->value = getEncryptedText(getStuff($value));
					}
					break;
				default:
					$registrar->ConfigParams[$control->name]->value = $value;
			}
		}

		try {
			$registrar->update();
			$registrar->ConfigParams->update();

			getCurrentSession()->Storage->infoMessages[] = blmsg("TRANS_SETTINGS_SAVED_SUCCESSFULLY");

			if ("wildwestdomains" == $registrar->module_name &&
				1 == $registrar->ConfigParams["moduleEnabled"]->value &&
				0 == $registrar->ConfigParams["moduleCertified"]->value
			) {
				return false;
			}
		} catch (ProductException $e) {
			$this->addErrorMessage(UIFactory()->ErrorMessage($e->getMessage()));
		}

		return ! $this->hasWarnings() && ! $this->hasErrors();
	}

	private $module_input_controls;
	private $input_registrar_name;
	private $input_registrar_enabled;
}

class EditDomainRegistrar extends MB_Action
{
	public function execute()
	{
		parent::execute();

		if (!isset($_REQUEST['configGroupID'])) {
			$this->addError(TRANS_ACTIONERROR, "No Registrar given.");
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}
		
		$configGroupID = bgpc("configGroupID");
		$back_url = action_url("registrars", "ShowRegistrars");
		$controller  = UIFactory()->PageController($back_url);
		$controller->addPage($edit_page = new EditDomainRegistrarPage($controller));
		$edit_page->title = blmsg("TRANS_REGISTRAR_OPTIONS");
		$edit_page->parent_title = blmsg("TRANS_MANAGE_SYSTEM");
		$edit_page->parent_url = $back_url;
		$controller->setValue("configGroupID", $configGroupID);
		$controller->processRequest();

		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}