<?php

class PurchaseManager {
	
	public static function purchaseDomains($request, $store, $cart, $plan, &$duplicatedDomains = array()) {
		
		$SLD = $request->getParameter('SLD');
		
		$registerTLDs = $request->getParameter('registerTLDs', array());
		$transferTLDs = $request->getParameter('transferTLDs', array());
		
		$alreadyPurchasedDomains = array();
		$domainPurchases = $cart->getDomainPurchases();
		foreach ($domainPurchases as $purchase) {
			$data = $purchase->getParameter('domainData');
			$alreadyPurchasedDomains[] = $data['name'];
		}
		
		$currentlyPurchasingDomains = array();
		if ($currentCartItem = $cart->getCurrentItem()) {
			foreach ($currentCartItem->getPurchases(true) as $purchase) {
				if ($purchase->getType() == BillingPlan::TYPE_DOMAIN) {
					$data = $purchase->getParameter('domainData');
					$currentlyPurchasingDomains[] = $data['name'];
				} else {
					if ($data = $purchase->getParameter('domainData')) {
						$alreadyPurchasedDomains[] = $data['name'];
					}
				}
			}
		}
		
		$count = 0;
		
		$purchases = array();
		
		foreach ($registerTLDs as $TLD) {
			$domainName = $SLD . '.' . $TLD;
			if (array_search($domainName, $alreadyPurchasedDomains) !== false) {
				$duplicatedDomains[] = $domainName;
				continue;
			}
			if (array_search($domainName, $currentlyPurchasingDomains) !== false) {
				$count++;
				continue;
			}
			$priceId = $request->getParameter('price_' . $TLD);
			$cycleId = $plan->getPriceById($priceId)->cycleID;
			
			$purchase = self::prepareDomainPurchase($plan->getId(), $cycleId, $priceId, BillingDomainPlan::PURCHASE_TYPE_REGISTER, $SLD, $TLD);
			if ($coupon = $plan->getPreappliedCoupon()) {
				$purchase->setCoupon($coupon);
			}
			
			$purchases[] = $purchase;
			$count++;
		}
		
		foreach ($transferTLDs as $TLD) {
			$domainName = $SLD . '.' . $TLD;
			if (array_search($domainName, $alreadyPurchasedDomains) !== false) {
				$duplicatedDomains[] = $domainName;
				continue;
			}
			if (array_search($domainName, $currentlyPurchasingDomains) !== false) {
				$count++;
				continue;
			}
			$priceId = $request->getParameter('price_' . $TLD);
			$cycleId = $plan->getPriceById($priceId)->cycleID;
			
			$purchase = self::prepareDomainPurchase($plan->getId(), $cycleId, $priceId, BillingDomainPlan::PURCHASE_TYPE_TRANSFER, $SLD, $TLD);
			if ($coupon = $plan->getPreappliedCoupon()) {
				$purchase->setCoupon($coupon);
			}
			
			$purchases[] = $purchase;
			$count++;
		}
		
		if ($count && !$duplicatedDomains) {
			$cartItem = $cart->getCurrentItem();
			foreach ($purchases as $purchase) {
				$cartItem->addPurchase($purchase);
			}
			return true;
		} else {
			return false;
		}
		
	}
	
	private static function prepareDomainPurchase($planId, $cycleId, $priceId, $type, $SLD, $TLD) {
		$domainName = $SLD . '.' . $TLD;
		$purchase = new Purchase($planId, $cycleId, $priceId, $domainName, BillingPlan::TYPE_DOMAIN);
		$purchase->addParameter('type', $type);
		$domainData = array(
			'SLD' => $SLD,
			'TLD' => $TLD,
			'name' => $domainName
		);
		$purchase->addParameter('domainData', $domainData);
		return $purchase;
	}
	
}