<?php

class BillingDomainPlan extends BillingPlan {
	
	const PURCHASE_TYPE_REGISTER = 'register';
	
	const PURCHASE_TYPE_TRANSFER = 'transfer';
	
	private $_domains = array();
	
	public function getDomains($storeId) {
		if (!isset($this->_domains[$storeId])) {
			$this->_domains[$storeId] = ResourceLocator::getInstance()->billingDomainService->getDomains($this->getId(), $storeId);
		}
		return $this->_domains[$storeId];
	}
	
	public function hasDomains($storeId) {
		return $this->getDomains($storeId) ? true : false;
	}
	
	public function getLowestCyclePrice($currencyId) {
		$amount = false;
		$result = null;
		foreach ($this->getVariants() as $variant) {
			foreach ($variant->getPricesByCurrency($currencyId) as $price) {
				if ($price->hasCycleFee() && (!$amount || $price->getCycleFee() < $amount)) {
					$amount = $price->getCycleFee();
					$result = $price;
				}
			}
		}
		return $result;
	}
	
	public function getLowestTransferPrice($currencyId) {
		$amount = false;
		$result = null;
		foreach ($this->getVariants() as $variant) {
			foreach ($variant->getPricesByCurrency($currencyId) as $price) {
				if ($price->hasTransferFee() && (!$amount || $price->getTransferFee() < $amount)) {
					$amount = $price->getTransferFee();
					$result = $price;
				}
			}
		}
		return $result;
	}
	
	public function hasTransferPrice($currencyId) {
		$result = false;
		foreach ($this->getVariants() as $variant) {
			foreach ($variant->getPricesByCurrency($currencyId) as $price) {
				if ($result = $price->hasTransferFee()) {
					break;
				}
			}
		}
		return $result;
	}
	
	public function mayHaveDomains() {
		return false;
	}
	
	public function mustHaveDomains() {
		return false;
	}
	
}