<?php

class BillingContactService extends BillingService {
	
	protected static $instance = null;
	
	public function getCreateCustomerForm($storeId) {
		$responce = $this->executeGetContactParams($storeId);
		if ($responce->hasErrors()) {
			throw new BillingServiceException('Can not retrieve create customer form for Store with id "' . $storeId . '"');
		} else {
			return new BillingContactInformationForm($responce->getData());
		}
	}
	
	public function collectCustomerForm($storeId, $values) {
		$responce = $this->executeCollectContactParams($storeId, $values);
		return new CollectFormResult($responce->getErrors(), $responce->getViolations(), $responce->getData());
	}
	
	public function login($login, $password) {
		$responce = $this->executeLogin($login, $password);
		if ($responce->hasErrors()) {
			throw new BillingObjectException('Cannot log in as "' . $login . '"');
		} else {
			$customer = new BillingCustomer($responce->getData());
			sfContext::getInstance()->getUser()->setAttribute('customer', $customer);
			$this->removeInvalidCoupons();
			return $customer;
		}
	}
	
	public function loginByShoppingSessionId($shoppingSessionId) {
		$responce = $this->executeLoginBySSID($shoppingSessionId);
		if ($responce->hasErrors()) {
			throw new BillingObjectException('Can not login by shopping session ID "' . $shoppingSessionId . '"');
		} else {
			$customer = new BillingCustomer($responce->getData());
			sfContext::getInstance()->getUser()->setAttribute('customer', $customer);
			$this->removeInvalidCoupons();
			return $customer;
		}
	}
	
	private function removeInvalidCoupons() {
		$cart = ResourceLocator::getInstance()->cart;
		foreach ($cart->getPurchases() as $purchase) {
			if (($coupon = $purchase->getCoupon()) && $coupon->couponNewOnly) {
				$purchase->unsetCoupon();
			}
		}
		foreach ($cart->getAddons() as $addon) {
			if (($coupon = $addon->getCoupon()) && $coupon->couponNewOnly) {
				$addon->unsetCoupon();
			}
		}
	}
	
	public static function getInstance() {
		if (self::$instance == null) {
			self::$instance = new self(self::prepareSoapClient('contact'));
		}
		return self::$instance;
	}
	
}