<?php

class BillingDomainService extends BillingService {
	
	protected static $instance = null;
	
	public function getDomains($planId, $storeId) {
		$domains = array();
		// domains prices we always retrieve anonymously,
		// because in current implementation no need to retrieve
		// prices for concrete client with concrete tax
		$responce = $this->executeGetDomainData($storeId, $planId, null);
		if (!$responce->hasErrors()) {
			foreach ($responce->getData() as $data) {
				$domains[] = new BillingDomain($data);
			}
		}
		return $domains;
	}
	
	public function checkWhois($SLD, $TLDs) {
		$responce = $this->executeCheckWhois($SLD, $TLDs);
		if ($responce->hasErrors()) {
			throw new BillingObjectException('Can not perform whois check for STD "' . $SLD . '"');
		} else {
			return $responce->getData();
		}
	}
	
	public function getDomainDetailsForm($storeId, $variantId, $SLD, $action) {
		
		if ($customer = sfContext::getInstance()->getUser()->getAttribute('customer')) {
			$clientId = $customer->getId();
			$customerData = array();
		} else {
			$clientId = null;
			$customerData = sfContext::getInstance()->getUser()->getAttribute('customerData');
		}
		
		$responce = $this->executeGetDomainParams($storeId, $variantId, $SLD, $action, $clientId, $customerData);
		if ($responce->hasErrors()) {
			throw new BillingServiceException('Can not retrieve domain details form for Store with id "' . $storeId . '"');
		} else {
			return new BillingDomainDetailsForm($responce->getData());
		}
	}
	
	public function collectDomainDetailsForm($storeId, $values, $variantId, $SLD, $action, $clientId = null) {
		$responce = $this->executeCollectDomainParams($storeId, $values, $variantId, $SLD, $action, $clientId);
		return new CollectFormResult($responce->getErrors(), $responce->getViolations(), $responce->getData());
	}
	
	public static function getInstance() {
		if (self::$instance == null) {
			self::$instance = new self(self::prepareSoapClient('domain'));
		}
		return self::$instance;
	}
	
}